"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _msw = require("msw");

var _react = require("@testing-library/react");

var _mocks = require("./helpers/mocks");

var _react2 = require("../react");

var _jsxRuntime = require("react/jsx-runtime");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

beforeAll(function () {
  _mocks.server.listen();
});
afterEach(function () {
  jest.clearAllMocks();

  _mocks.server.resetHandlers();
});
afterAll(function () {
  _mocks.server.close();
});
test("it won't allow to fetch the session in isolation without a session context", function () {
  function App() {
    (0, _react2.useSession)();
    return null;
  }

  jest.spyOn(console, "error");
  console.error.mockImplementation(function () {});
  expect(function () {
    return (0, _react.render)((0, _jsxRuntime.jsx)(App, {}));
  }).toThrow("useSession must be wrapped in a SessionProvider");
  console.error.mockRestore();
});
test("fetches the session once and re-uses it for different consumers", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee() {
  var sessionRouteCall;
  return _regenerator.default.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          sessionRouteCall = jest.fn();

          _mocks.server.use(_msw.rest.get("/api/auth/session", function (req, res, ctx) {
            sessionRouteCall();
            res(ctx.status(200), ctx.json(_mocks.mockSession));
          }));

          (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {}));
          expect(_react.screen.getByTestId("session-consumer-1")).toHaveTextContent("loading");
          expect(_react.screen.getByTestId("session-consumer-2")).toHaveTextContent("loading");
          _context.next = 7;
          return (0, _react.waitFor)(function () {
            expect(sessionRouteCall).toHaveBeenCalledTimes(1);

            var session1 = _react.screen.getByTestId("session-consumer-1").textContent;

            var session2 = _react.screen.getByTestId("session-consumer-2").textContent;

            expect(session1).toEqual(session2);
          });

        case 7:
        case "end":
          return _context.stop();
      }
    }
  }, _callee);
})));
test("when there's an existing session, it won't initialize as loading", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee2() {
  var sessionRouteCall;
  return _regenerator.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          sessionRouteCall = jest.fn();

          _mocks.server.use(_msw.rest.get("/api/auth/session", function (req, res, ctx) {
            sessionRouteCall();
            res(ctx.status(200), ctx.json(_mocks.mockSession));
          }));

          (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {
            session: _mocks.mockSession
          }));
          _context2.t0 = expect;
          _context2.next = 6;
          return _react.screen.findByTestId("session-consumer-1");

        case 6:
          _context2.t1 = _context2.sent;
          (0, _context2.t0)(_context2.t1).not.toHaveTextContent("loading");
          expect(_react.screen.getByTestId("session-consumer-2")).not.toHaveTextContent("loading");
          expect(sessionRouteCall).not.toHaveBeenCalled();

        case 10:
        case "end":
          return _context2.stop();
      }
    }
  }, _callee2);
})));

function ProviderFlow(_ref3) {
  var _ref3$options = _ref3.options,
      options = _ref3$options === void 0 ? {} : _ref3$options;
  return (0, _jsxRuntime.jsxs)(_react2.SessionProvider, _objectSpread(_objectSpread({}, options), {}, {
    children: [(0, _jsxRuntime.jsx)(SessionConsumer, {}), (0, _jsxRuntime.jsx)(SessionConsumer, {
      testId: "2"
    })]
  }));
}

function SessionConsumer(_ref4) {
  var _ref4$testId = _ref4.testId,
      testId = _ref4$testId === void 0 ? 1 : _ref4$testId;

  var _useSession = (0, _react2.useSession)(),
      session = _useSession.data,
      status = _useSession.status;

  return (0, _jsxRuntime.jsx)("div", {
    "data-testid": "session-consumer-".concat(testId),
    children: status === "loading" ? "loading" : JSON.stringify(session)
  });
}