"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = oAuthCallback;

var _client = require("./client");

var _oauth = require("./oauth1");

var _stateHandler = require("./state-handler");

var _pkceHandler = require("./pkce-handler");

var _errors = require("../../../lib/errors");

async function oAuthCallback(req, res) {
  var _provider$version;

  const {
    provider,
    logger
  } = req.options;

  try {
    var _req$body$error;

    const error = (_req$body$error = req.body.error) !== null && _req$body$error !== void 0 ? _req$body$error : req.query.error;

    if (error) {
      throw new Error(error);
    }
  } catch (error) {
    logger.error("OAUTH_CALLBACK_HANDLER_ERROR", error, req.body, provider.id);
    throw error;
  }

  if ((_provider$version = provider.version) !== null && _provider$version !== void 0 && _provider$version.startsWith("1.")) {
    try {
      const client = await (0, _oauth.oAuth1Client)(req.options);
      const {
        oauth_token,
        oauth_verifier
      } = req.query;
      const tokens = await client.getOAuthAccessToken(oauth_token, null, oauth_verifier);
      const profileData = await client.get(provider.profileUrl, tokens.accessToken, tokens.refreshToken);
      return getProfile({
        profile: profileData,
        tokens,
        provider,
        logger
      });
    } catch (error) {
      logger.error("OAUTH_V1_GET_ACCESS_TOKEN_ERROR", error);
      throw error;
    }
  }

  try {
    var _ref, _req$body$user;

    const client = await (0, _client.openidClient)(req.options);
    const params = client.callbackParams(req);
    const checks = {
      code_verifier: await (0, _pkceHandler.usePKCECodeVerifier)(req, res),
      state: (0, _stateHandler.getState)(req)
    };
    let profile;
    let tokens;

    if (provider.idToken) {
      tokens = await client.callback(provider.callbackUrl, params, checks);
      profile = tokens.claims();
    } else {
      tokens = await client.oauthCallback(provider.callbackUrl, params, checks);
      profile = await client.userinfo(tokens);
    }

    profile.user = JSON.parse((_ref = (_req$body$user = req.body.user) !== null && _req$body$user !== void 0 ? _req$body$user : req.query.user) !== null && _ref !== void 0 ? _ref : null);
    return getProfile({
      profile,
      provider,
      tokens,
      logger
    });
  } catch (error) {
    logger.error("OAUTH_CALLBACK_ERROR", error, provider.id);
    throw new _errors.OAuthCallbackError(error);
  }
}

async function getProfile({
  profile: OAuthProfile,
  tokens,
  provider,
  logger
}) {
  try {
    var _profile$email$toLowe, _profile$email;

    logger.debug("PROFILE_DATA", OAuthProfile);
    const profile = await provider.profile(OAuthProfile, tokens);
    profile.email = (_profile$email$toLowe = (_profile$email = profile.email) === null || _profile$email === void 0 ? void 0 : _profile$email.toLowerCase()) !== null && _profile$email$toLowe !== void 0 ? _profile$email$toLowe : null;
    return {
      profile,
      account: {
        provider: provider.id,
        type: provider.type,
        id: profile.id,
        ...tokens
      },
      OAuthProfile
    };
  } catch (error) {
    logger.error("OAUTH_PARSE_PROFILE_ERROR", error, OAuthProfile);
    return {
      profile: null,
      account: null,
      OAuthProfile
    };
  }
}