"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getAuthorizationUrl;

var _client = require("../oauth/client");

var _oauth = require("../oauth/oauth1");

var _logger = _interopRequireDefault(require("../../../lib/logger"));

var _stateHandler = require("../oauth/state-handler");

var _pkceHandler = require("../oauth/pkce-handler");

async function getAuthorizationUrl(req, res) {
  var _req$query, _provider$version;

  const provider = req.options.provider;
  (_req$query = req.query) === null || _req$query === void 0 ? true : delete _req$query.nextauth;
  const params = {
    scope: provider.scope,
    ...provider.params,
    ...req.query
  };

  if ((_provider$version = provider.version) !== null && _provider$version !== void 0 && _provider$version.startsWith("1.")) {
    try {
      const tokens = await _oauth.oAuth1Client.getOAuthRequestToken(params);
      const url = `${provider.authorizationUrl}?${new URLSearchParams({
        oauth_token: tokens.oauth_token,
        oauth_token_secret: tokens.oauth_token_secret,
        ...tokens.params
      })}`;

      _logger.default.debug("GET_AUTHORIZATION_URL", url);

      return url;
    } catch (error) {
      _logger.default.error("GET_AUTHORIZATION_URL_ERROR", error);

      throw error;
    }
  }

  const client = await (0, _client.openidClient)(req.options);
  const pkce = await (0, _pkceHandler.createPKCE)(req, res);
  return client.authorizationUrl({ ...params,
    ...pkce,
    state: (0, _stateHandler.createState)(req)
  });
}